@@ -7,16 +7,25 @@ modulename = 'csoundengine.engine'
 logger = logging.getLogger('csoundengine')
 
 
+def setLoggingLevel(level: str) -> None:
+    """
+    Utility to set the logging level of csoundengine
+    """
+    level = level.upper()
+    logging.basicConfig(level=level)
+    logger.setLevel(level)
+
+
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ #
 #                  CONFIG                   #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ #
 
-config = ConfigDict(modulename.replace(".", ":"))
+config = ConfigDict(modulename)
 _ = config.addKey
 
 
 def _validateBackend(cfg: dict, key:str, s: str) -> bool:
-    platform = key.split(".")[0]
+    platform = key.split("_")[0]
     possibleBackends = csoundlib.getAudioBackendNames(available=False, platform=platform)
     for backend in (b.strip() for b in s.split(',')):
         if backend not in possibleBackends:
@@ -29,7 +38,7 @@ def _validateBackend(cfg: dict, key:str, s: str) -> bool:
 _('sr', 0,
   choices=(0, 22050, 44100, 48000, 88200, 96000),
   doc='samplerate - 0=default sr for the backend')
-_('rec.sr', 44100,
+_('rec_sr', 44100,
   choices=(44100, 48000, 88200, 96000, 192000),
   doc='default samplerate when rendering')
 _('nchnls', 0,
@@ -41,23 +50,23 @@ _('nchnls_i', 0,
 _('ksmps', 64,
   choices=(16, 32, 64, 128, 256),
   doc="corresponds to csound's ksmps")
-_('rec.ksmps', 64,
+_('rec_ksmps', 64,
   choices=(16, 32, 64, 128, 256),
   doc="samples per cycle when rendering")
-_('rec.sample_format', 'float',
+_('rec_sample_format', 'float',
   choices=(16, 24, 32, 'float'),
   doc="Sample format used when rendering")
 _('buffersize', 0,
   doc="-b value. 0=determine buffersize depending on ksmps & backend")
 _('numbuffers', 0,
   doc="determines the -B value as a multiple of the buffersize. 0=auto")
-_('linux.backend', 'jack, pulse, pa_cb',
+_('linux_backend', 'jack, pulse, pa_cb',
   doc="a comma separated list of backends (possible backends: jack, pulse, pa_cb, alsa)",
   validatefunc=_validateBackend),
-_('macos.backend', 'pa_cb',
+_('macos_backend', 'pa_cb',
   doc="a comma separated list of backends (possible backends: pa_cb, auhal)",
   validatefunc=_validateBackend),
-_('windows.backend', 'pa_cb',
+_('windows_backend', 'pa_cb',
   doc="a comma separated list of backends (possible backends: pa_cb, pa_bl)",
   validatefunc=_validateBackend)
 _('A4', 442,
@@ -87,7 +96,7 @@ _('associated_table_min_size', 16,
 _('num_audio_buses', 64,
   doc="Num. of audio buses in an Engine/Session")
 _('num_control_buses', 512,
-  doc="Num. of contro buses in an Engine/Session")
+  doc="Num. of control buses in an Engine/Session")
 _('html_theme', 'light',
   choices={'dark', 'light'},
   doc="Style to use when displaying syntax highlighting")
@@ -95,6 +104,21 @@ _('html_args_fontsize', '12px',
   doc="Font size used for args when outputing html (in jupyter)")
 _('synth_repr_max_args', 12,
   doc="Max. number of pfields shown when in a synth's repr")
+_('jupyter_synth_repr_stopbutton', True,
+  doc='When running inside a jupyter notebook, display a stop button'
+      'for Synths and SynthGroups')
+_('jupyter_synth_repr_interact', True,
+  doc='When inside jupyter, add interactive widgets if a synth has'
+      'named parameters')
+_('jupyter_instr_repr_show_code', True,
+  doc='Show code when displaying an Instr inside jupyter')
+_('ipython_load_magics_at_startup', True,
+  doc='Load csoundengine.magic at startup when inside ipython. If False, magics can '
+      'still be loaded via `%load_ext csoundengine.magic`')
+_('magics_print_info', True,
+  doc='Print some informative information when the csounengine.magic extension is loaded')
+_('jupyter_slider_width', '80%',
+  doc='CSS Width used by an interactive slider in jupyter')
 
 assert 'num_control_buses' in config.default
 config.load()
