@@ -291,17 +291,18 @@ class Instr:
         if not pargs:
             return ""
         if self.pargsDefaultValues:
-            return ", ".join(f"p{i}:{pname}={self.pargsDefaultValues.get(i, 0)}"
+            return ", ".join(f"{pname}:{i}={self.pargsDefaultValues.get(i, 0)}"
                              for i, pname in sorted(pargs.items()) if i != 4)
         else:
             return ", ".join(
-                    f"p{i}: {pname}" for i, pname in sorted(pargs.items()) if i != 4)
+                    f"{pname}:{i}" for i, pname in sorted(pargs.items()) if i != 4)
 
     def _repr_html_(self) -> str:
         style = jupytertools.defaultStyle
         parts = [f'Instr <strong style="color:{style["name.color"]}">{self.name}</strong><br>']
         if self.pargsIndexToName and len(self.pargsIndexToName) > 1:
             indexes = list(self.pargsIndexToName.keys())
+            indexes.sort()
             if 4 in indexes:
                 indexes.remove(4)
             groups = iterlib.splitInChunks(indexes, 5)
@@ -318,7 +319,8 @@ class Instr:
                 parts.append(line)
         if self.tabledef:
             parts.append(f'&nbsp&nbsp&nbsp&nbsptabargs = <code>{self.tabledef}</code>')
-        parts.append(csoundengine.csoundlib.highlightCsoundOrc(self.body))
+        if config['jupyter_instr_repr_show_code']:
+            parts.append(csoundengine.csoundlib.highlightCsoundOrc(self.body))
         return "\n".join(parts)
 
 
