@@ -4,19 +4,19 @@ import numpy as np
 from .config import logger, config
 from . import internalTools
 from emlib import iterlib
+import emlib.misc
 import weakref as _weakref
 from . import jupytertools
 
-from typing import TYPE_CHECKING, Union as U, Optional as Opt, KeysView, Dict, List, Set
-
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from typing import *
     from .engine import Engine
     from .instr import Instr
     from .paramtable import ParamTable
     from .session import Session
 
-
 __all__ = ['AbstrSynth', 'Synth', 'SynthGroup']
 
 
@@ -50,7 +50,6 @@ class AbstrSynth:
         """ Is this synth playing? """
         raise NotImplementedError()
 
-
     def finished(self) -> bool:
         """ Has this synth ceased to play? """
         raise NotImplementedError()
@@ -100,7 +99,7 @@ class AbstrSynth:
         """
         raise NotImplementedError()
 
-    def get(self, slot: U[int, str], default: float = None) -> Opt[float]:
+    def get(self, slot: Union[int, str], default: float = None) -> Optional[float]:
         """
         Get the value of a named parameter
 
@@ -115,7 +114,7 @@ class AbstrSynth:
         """
         raise NotImplementedError()
 
-    def tableParams(self) -> Opt[Set[str]]:
+    def tableParams(self) -> Optional[Set[str]]:
         """
         Return a seq. of all named parameters
 
@@ -136,7 +135,7 @@ class AbstrSynth:
         """ Does this synth have an associated parameter table?"""
         raise NotImplementedError()
 
-    def automateTable(self, param: str, pairs: U[List[float], np.ndarray],
+    def automateTable(self, param: str, pairs: Union[List[float], np.ndarray],
                       mode="linear", delay=0., overtake=False) -> None:
         """
         Automate a table parameter. Time stamps are relative to the start
@@ -155,14 +154,18 @@ class AbstrSynth:
         """
         raise NotImplementedError()
 
-    def namedPfields(self) -> Opt[Set[str]]:
+    def namedPfields(self) -> Optional[Set[str]]:
         """
         Returns a set of all named pfields
         """
         raise NotImplementedError()
 
-    def automatep(self, param: U[int, str], pairs: U[List[float], np.ndarray],
-                  mode="linear", delay=0., overtake=False) -> AbstrSynth:
+    def automatep(self,
+                  param: Union[int, str],
+                  pairs: Union[List[float], np.ndarray],
+                  mode="linear",
+                  delay=0.,
+                  overtake=False) -> AbstrSynth:
         """
         Automate the value of a pfield.
 
@@ -276,7 +279,7 @@ class Synth(AbstrSynth):
         self.startTime: float = starttime or time.time()
         self.dur: float = dur
         self.pargs: List[float] = pargs
-        self.table: Opt[ParamTable] = table
+        self.table: Optional[ParamTable] = table
         self.synthGroup = synthgroup
         self._playing: bool = True
 
@@ -289,51 +292,64 @@ class Synth(AbstrSynth):
             f'{playstr} <strong style="color:{style["name.color"]}">'
             f'{self.instr.name}</strong>:{self.synthid:.4f}',
             ]
-        if self.table:
+        if self.table is not None:
             parts.append(self.table._mappingRepr())
         if self.pargs:
             i2n = self.instr.pargsIndexToName
             argsstrs = []
             pargs = self.pargs[0:]
+            if any(arg.startswith('k') for arg in self.instr.pargsNameToIndex):
+                maxi = max(i+4 for i, n in i2n.items()
+                           if n.startswith('k'))
+            for i, parg in enumerate(pargs, start=4):
+                if i > maxi:
+                    argsstrs.append("...")
+                    break
+                name = i2n.get(i)
+                if name:
+                    s = f"<b>{name}</b>:{i}=<code>{parg:.6g}</code>"
+                else:
+                    # s = f"<b>p{i + 4}</b>=<code>{parg:.6g}</code>"
+                    s = f"<b>{i}</b>=<code>{parg:.6g}</code>"
+                argsstrs.append(s)
+            argsstr = " ".join(argsstrs)
+            argsstr = fr'<span style="font-size:{argsfontsize};">{argsstr}</span>'
+            parts.append(argsstr)
+        # return '<span style="font-size:12px;">∿(' + ', '.join(parts) + ')</span>'
+        return '<span style="font-size:12px;">Synth('+', '.join(parts)+')</span>'
+
+    def _repr_html_(self) -> str:
+        if emlib.misc.inside_jupyter():
+            if config['jupyter_synth_repr_stopbutton']:
+                jupytertools.displayButton("Stop", self.stop)
+            if config['jupyter_synth_repr_interact'] and self.pargs:
+                pass
+        return f"<p>{self._html()}</p>"
+
+    def __repr__(self):
+        playstr = _synthStatusIcon[self.playStatus()]
+        parts = [f'{playstr} {self.instr.name}:{self.synthid}']
+        if self.table is not None:
+            parts.append(self.table._mappingRepr())
+        if self.pargs:
+            maxi = config['synth_repr_max_args']
+            i2n = self.instr.pargsIndexToName
+            maxi = max((i for i, name in i2n.items() if name.startswith("k")),
+                           default=maxi)
+            argsstrs = []
+            pargs = self.pargs[0:]
             for i, parg in enumerate(pargs, start=0):
                 if i > maxi:
                     argsstrs.append("...")
                     break
-                name = i2n.get(i + 4)
-                if name:
-                    s = f"<b>{name}</b>:{i + 4}=<code>{parg:.6g}</code>"
-                else:
-                    # s = f"<b>p{i + 4}</b>=<code>{parg:.6g}</code>"
-                    s = f"<b>{i + 4}</b>=<code>{parg:.6g}</code>"
-                argsstrs.append(s)
-            argsstr = " ".join(argsstrs)
-            argsstr = fr'<span style="font-size:{argsfontsize};">{argsstr}</span>'
-            parts.append(argsstr)
-        return '<span style="font-size:12px;">∿(' + ', '.join(parts) + ')</span>'
-
-    def _repr_html_(self) -> str:
-        return f"<p>{self._html()}</p>"
-
-    def __repr__(self):
-        playstr = _synthStatusIcon[self.playStatus()]
-        parts = [f'{playstr} {self.instr.name}', f'id={self.synthid}']
-        if self.table:
-            parts.append(self.table._mappingRepr())
-        if self.pargs:
-            i2n = self.instr.pargsIndexToName
-            argsstrs = []
-            pargs = self.pargs[0:]
-            for i, parg in enumerate(pargs, start=0):
-                if i > 6:
-                    argsstrs.append("...")
-                    break
                 name = i2n.get(i+4)
                 if name:
-                    s = f"p{i+4}:{name}={parg:.8g}"
+                    s = f"{name}:{i+4}={parg:.6g}"
+                    #s = f"p{i+4}:{name}={parg:.8g}"
                 else:
-                    s = f"p{i+4}={parg:.8g}"
+                    s = f"p{i+4}={parg:.6g}"
                 argsstrs.append(s)
-            argsstr = ", ".join(argsstrs)
+            argsstr = " ".join(argsstrs)
             parts.append(argsstr)
         lines = ["Synth(" + ", ".join(parts) + ")"]
         # add a line for k- pargs
@@ -374,12 +390,12 @@ class Synth(AbstrSynth):
     def finished(self) -> bool:
         return self.playStatus() == 'stopped'
 
-    def tableState(self) -> Opt[Dict[str, float]]:
+    def tableState(self) -> Optional[Dict[str, float]]:
         if self.table is None:
             return None
         return self.table.asDict()
 
-    def tableParams(self) -> Opt[Set[str]]:
+    def tableParams(self) -> Optional[Set[str]]:
         if self.table is None:
             return None
         return set(self.table.mapping.keys())
@@ -410,7 +426,8 @@ class Synth(AbstrSynth):
                 for key, value in kws.items():
                     self.table[key] = value
 
-    def get(self, slot: U[int, str], default: float = None) -> Opt[float]:
+    def get(self, slot: Union[int, str], default: float = None
+            ) -> Optional[float]:
         if not self._playing:
             logger.error("Synth not playing")
             return
@@ -421,7 +438,7 @@ class Synth(AbstrSynth):
 
         return self.table.get(slot, default)
 
-    def namedPfields(self) -> Opt[Set[str]]:
+    def namedPfields(self) -> Optional[Set[str]]:
         name2idx = self.instr.pargsNameToIndex
         return set(name2idx.keys()) if name2idx else None
 
@@ -457,7 +474,6 @@ class Synth(AbstrSynth):
             # Can we just modify the scheduled value?
             return
 
-
         # most common use: just one pair
         if not kws and len(args) == 2:
             k = args[0]
@@ -479,11 +495,65 @@ class Synth(AbstrSynth):
         pairs = iterlib.flatdict(pairsd)
         self.engine.setp(self.synthid, *pairs, delay=delay)
 
+    def ui(self, **specs: Dict[str, Tuple[float, float]]) -> None:
+        """
+        Modify dynamic (named) arguments through an interactive user-interface
+
+        If run inside a jupyter notebook, this method will create embedded widgets
+        to control the values of the dynamic pfields of an event
+
+        Args:
+            **specs: a dict mapping named arg to a tuple (minvalue, maxvalue)
+
+        Example
+        =======
+
+        .. code::
+
+            # Inside jupyter
+            from csoundengine import *
+            s = Engine().session()
+            s.defInstr('vco', r'''
+              |kmidinote, kampdb=-12, kcutoff=3000, kres=0.9|
+              kfreq = mtof:k(kmidinote)
+              asig = vco2:a(ampdb(kampdb), kfreq)
+              asig = moogladder2(asig, kcutoff, kres)
+              asig *= linsegr:a(0, 0.1, 1, 0.1, 0)
+              outs asig, asig
+            ''')
+            synth = s.sched('vco', kmidinote=67)
+            # Specify the ranges for some sliders. All named parameters
+            # are assigned a widget
+            synth.ui(kampdb=(-48, 0), kres=(0, 1))
+
+        .. figure:: ../assets/synthui.png
+
+        """
+        if self.playStatus() == 'future':
+            return
+        from . import interact
+        pairs = list(self.instr.pargsIndexToName.items())
+        pairs.sort()
+        pargindexes, pargnames = zip(*pairs)
+        pvalues = [self.engine.getp(self.synthid, idx) for idx in pargindexes]
+        paramspecs = {}
+        for idx, pargname, value in zip(pargindexes, pargnames, pvalues):
+            if pargname in specs:
+                minval, maxval = specs[pargname]
+            else:
+                minval, maxval = interact._guessRange(value, pargname)
+            paramspecs[idx] = interact.ParamSpec(pargname,
+                                                      minvalue=minval,
+                                                      maxvalue=maxval,
+                                                      startvalue=value)
+        return interact.interactPargs(self.engine, self.synthid, specs=paramspecs)
+
+
     def hasParamTable(self) -> bool:
         """ Returns True if this synth has an associated parameter table """
         return self.table is not None
 
-    def automateTable(self, param: str, pairs: U[List[float], np.ndarray],
+    def automateTable(self, param: str, pairs: Union[List[float], np.ndarray],
                       mode="linear", delay=0., overtake=False) -> float:
         if not self.table:
             raise RuntimeError(
@@ -496,7 +566,7 @@ class Synth(AbstrSynth):
         return self.engine.automateTable(self.table.tableIndex, paramidx, pairs,
                                          mode=mode, delay=delay, overtake=overtake)
 
-    def automatep(self, param: U[int, str], pairs: U[List[float], np.ndarray],
+    def automatep(self, param: Union[int, str], pairs: Union[List[float], np.ndarray],
                   mode="linear", delay=0., overtake=False) -> float:
         if isinstance(param, str):
             pidx = self.instr.pargIndex(param)
@@ -518,6 +588,7 @@ class Synth(AbstrSynth):
             self.session.unsched(self.synthid, delay=delay)
 
 
+
 class SynthGroup(AbstrSynth):
     """
     A SynthGroup is used to control multiple synths created
@@ -568,8 +639,12 @@ class SynthGroup(AbstrSynth):
                 out.update(namedPargs)
         return out
 
-    def automatep(self, param: U[int, str], pairs: U[List[float], np.ndarray],
-                  mode="linear", delay=0., overtake=False) -> List[float]:
+    def automatep(self,
+                  param: Union[int, str],
+                  pairs: Union[List[float], np.ndarray],
+                  mode="linear",
+                  delay=0.,
+                  overtake=False) -> List[float]:
         eventids = []
         for synth in self.synths:
             if synth.table and param in synth.tableParams():
@@ -580,7 +655,7 @@ class SynthGroup(AbstrSynth):
     def hasParamTable(self) -> bool:
         return any(s.hasParamTable() is not None for s in self.synths)
 
-    def tableState(self) -> Opt[Dict[str, float]]:
+    def tableState(self) -> Optional[Dict[str, float]]:
         dicts = []
         for s in self.synths:
             d = s.tableState()
@@ -593,7 +668,60 @@ class SynthGroup(AbstrSynth):
             out.update(d)
         return out
 
+    def _uniqueInstr(self) -> bool:
+        instr0 = self.synths[0].instr
+        return all(synth.instr == instr0 for synth in self.synths if synth.playing())
+
+    def _htmlTable(self) -> Optional[str]:
+        synth0 = self.synths[0]
+        instr0 = synth0.instr
+        if any(synth.instr != instr0 for synth in self.synths):
+            return
+        colnames = ["p1", "start", "dur"]
+        rows = [[] for _ in self.synths]
+        for row, synth in zip(rows, self.synths):
+            row.append(f'{synth.synthid} <b>{_synthStatusIcon[synth.playStatus()]}</b>')
+            row.append("%.3f"%(synth.startTime-time.time()))
+            row.append("%.3f"%synth.dur)
+        if synth0.table is not None:
+            keys = list(synth0.table.mapping.keys())
+            colnames.extend(synth0.table.mapping.keys())
+            for row, synth in zip(rows, self.synths):
+                if synth.playStatus() != 'stopped':
+                    values = synth.table.array[:len(keys)]
+                    for value in values:
+                        row.append(f'<code>{value}</code>')
+                else:
+                    row.extend(["-"] * len(keys))
+
+        if synth0.pargs:
+            maxi = config['synth_repr_max_args']
+            i2n = instr0.pargsIndexToName
+            maxi = max((i for i, name in i2n.items() if name.startswith("k")),
+                       default=maxi)
+            for i, parg in enumerate(synth0.pargs):
+                if i > maxi:
+                    colnames.append("...")
+                name = i2n.get(i+4)
+                if name:
+                    colnames.append(f"{i+4}:{name}")
+                else:
+                    colnames.append(str(i+4))
+            for row, synth in zip(rows, self.synths):
+                row.extend("%.5g"%parg for parg in synth.pargs[:maxi])
+                if len(synth.pargs) > maxi:
+                    row.append("...")
+
+        return emlib.misc.html_table(rows, headers=colnames)
+
     def _repr_html_(self) -> str:
+        if config['jupyter_synth_repr_stopbutton'] and emlib.misc.inside_jupyter():
+            jupytertools.displayButton("Stop", self.stop)
+        if self._uniqueInstr():
+            instrcol = jupytertools.defaultStyle["name.color"]
+            header = f'SynthGroup - <b>{len(self.synths)}</b> synths, ' \
+                     f'instr: <strong style="color:{instrcol}">{self.synths[0].instr.name}</strong>'
+            return header + self._htmlTable()
         parts = ['<p>SynthGroup']
         for s in self.synths:
             html = s._html()
@@ -622,7 +750,7 @@ class SynthGroup(AbstrSynth):
         for synth in self.synths:
             synth.set(*args, delay=delay, **kws)
 
-    def get(self, idx: U[int, str], default=None) -> List[Opt[float]]:
+    def get(self, idx: Union[int, str], default=None) -> List[Optional[float]]:
         """
         Get the value of a tabarg
 
